/*
    This file is part of Fate.

    Copyright (c) 2007 SUSE LINUX Products GmbH
    
    Author: André Duffeck <aduffeck@suse.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#ifndef XCLVALIDATOR_H
#define XCLVALIDATOR_H

#include <qvaluelist.h>
#include <qdom.h>

#include "featurecheck.h"
#include "xclobject.h"

/**
  * The XCLValidator allows to perform integrity checks on an XML document.
  *
  * It therefor reads an XCL description document and performs the specified
  * checks on the given XML document. The results are returned as XML, too.
  * 
  * XCL allows to make checks dependent on some conditions (<xcl:condition>), that means that the
  * check is only done if the condition is fulfilled. Conditions can be logically 
  * grouped with the tags <xcl:all>, <xcl:oneOf> and <xcl:none> which correspond to 
  * the logical AND, OR and NOT. Those tags can also be nested which allows to 
  * express every condition.
  * 
  * If a condition is fulfilled or no condition is set for the check, the tests in
  * <xcl:claim> are performed. Again, the tests can be used with the logical tags 
  * described above. The most simple kind of test is comparing two values:
  * 
  *   \code<xcl:test>status/evaluation/owner/role = 'projectmanager'</xcl:test>\endcode
  * 
  * This test evaluates the content of the node "status/evaluation/owner/role" and 
  * returns true if it is "projectmanager". Other valid comparators besides "=" are
  * "!=", "<", "<=", ">" and ">=".
  * 
  * There are also some special functions that can be used on nodes, e.g. "length(node)" 
  * which returns the number of chars of the node's content.
  * 
  * Another special test is the <xcl:hasElement> test, which traverses the XML tree and 
  * returns true if it finds a node which fulfills the given condition.
  * 
  * Example XCL document:\code
  * <xcl:constraints>
  *  <xcl:constraint object="productcontext" mandatory="true">
  *   <xcl:condition>
  *    <xcl:all>
  *     <xcl:test>status/evaluation</xcl:test>
  *     <xcl:test>status/evaluation/owner/role = 'productmanager'</xcl:test>
  *    </xcl:all>
  *   </xcl:condition>
  *   <xcl:claim>
  *    <xcl:all>
  *     <xcl:hasElement path="../actor">
  *       <xcl:all>
  *         <xcl:test>role = 'productmanager'</xcl:test>
  *       </xcl:all>
  *     </xcl:hasElement>
  *    </xcl:all>
  *   </xcl:claim>
  *   <xcl:description>The feature is in state 'Evaluation by product manager'
  *        and therefore should have productmanager defined</xcl:description>
  *  </xcl:constraint>
  * </xcl:constraints>\endcode
  */
class XCLValidator
{
  public:
    enum Mode { OneOf, All, None };

    XCLValidator();

      /**
     * Sets the XCL document containing the check descriptions
     * @param doc the XCL document
     */
    bool setXCLDocument( const QString &doc );

      /**
     * Performs the checks on the XML document 'xml'
     * @param xml
     */
    FeatureCheck::Map validate( const QDomNode &xml, const XCLObject *o = 0 );

  protected:
      /**
     * Apply one constraint on a node
     * @param constraint the constraint node
     * @param e the xml node
     */
    bool applyConstraint( const QDomNode &constraint, const QDomNode &e );

      /**
     * Perform one specific test on a node
     * @param test the test node
     * @param e the xml node
     */
    bool performTest( const QDomNode &test, const QDomNode &e );

      /**
     * Perform a "hasNode" test on a node
     * @param test the hasNode test node
     * @param e the xml node
     */
    bool performHasElement( const QDomNode &test, const QDomNode &e );

      /**
     * Perform a "validDTD" test on a node
     * @param test the validRichtext test node
     * @param e the xml node
     */
    bool performValidDTD( const QDomNode &test, const QDomNode &e );

      /**
     * Returns true if the node meets the condition
     * @param test the condition node
     * @param e the xml node
     */
    bool test( const QDomNode &condition, const QDomNode &e );

      /**
     * Returns the value of a node
     * @param path the node of which the value should be returned
     * @param e the xml node
     */
    QString getValue( const QDomNode &e, const QString &path );

      /**
     * Returns the xml representation of a node including the child tags
     * and so on
     * @param path the node of which the value should be returned
     * @param e the xml node
     */
    QString getPlainValue( const QDomNode &e, const QString &path );

      /**
     * Returns the integer value of a node
     * @param path the node of which the value should be returned
     * @param e the xml node
     */
    int getIntValue( const QDomNode &e, const QString &path );

      /**
     * Returns true, if the node has a child named 'name'
     * @param n the node that 'name' is related to
     * @param name name or path of the child
     */
    bool hasChild( const QDomNode &n, const QString &name );

      /**
     * Returns all child nodes named 'name'
     * @param n the node that 'name' is related to
     * @param name name or path to the node in question
     */
    QDomNode getChild( const QDomNode &n, const QString &name );

      /**
     * Returns all child nodes named 'name'
     * @param n the node that 'name' is related to
     * @param name name or path to the nodes in question
     */
    QValueList<QDomNode> getChildren( const QDomNode &n, const QString &name );

      /**
     * Returns true, if the specified node denotes a number,
     * e.g. 5 or length(node)
     * @param n the node that 'name' is related to
     * @param name name or path to the node in question
     */
    bool isNumber( const QDomNode &n, const QString &name );

      /**
     * Returns true, if the specified node denotes a string
     * @param n the node that 'name' is related to
     * @param name name or path to the node in question
     */
    bool isString( const QDomNode &n, const QString &name );

  private:
    QString mBasePath;
    QMap< QString, QString > mVariables;
    QDomDocument mXCLDoc;
    FeatureCheck *mCurrentCheck;
    const XCLObject *mXCLObject;
};

#endif
